@@ -46,6 +46,16 @@ class QueueManager(object):
         else:
             raise Exception("Invalid dsn argument given (%s)." % str(dsn))
 
+    def fetchone(self, cur):
+        if   callable(getattr(cur, 'fetchone', None)): # psycopg2 or sql-alchemy
+            return cur.fetchone()
+        raise ValueError("Unknown how to fetch an item from specified session.")
+
+    def fetchall(self, cur):
+        if   callable(getattr(cur, 'fetchall', None)): # psycopg2 or sql-alchemy
+            return cur.fetchall()
+        raise ValueError("Unknown how to fetch items from specified session.")
+
     @contextmanager
     def session(self, other_sess):
         conn = None
@@ -60,8 +70,8 @@ class QueueManager(object):
                 yield (conn, cur)
             except:
                 if conn and cur and (self.invoking_queue_id != None):
-                    cur.execute(self.report_sql % (self.invoking_queue_id,))
-                    res = cur.fetchone()
+                    executed = cur.execute(self.report_sql % (self.invoking_queue_id,))
+                    res = self.fetchone(cur if (not executed) else executed)
                     if res and res[0]:
                         conn.commit()
                 raise
@@ -85,7 +95,7 @@ create table %s (
 );
 create index %s_tag_idx         on %s(tag);
 create index %s_created_at_idx  on %s(created_at);
-create index %s_schedule        on %s(schedule);
+create index %s_schedule_idx    on %s(schedule);
 """ % (n, self.data_length, n, n, n, n, n, n)
         self.drop_table_sql = """
 drop table %s;
@@ -155,10 +165,10 @@ listen %s;
     def enqueue(self, tag, data, other_sess = None, schedule = None):
         tag, data = (self.check_tag(tag), self.sanitize(self.serializer(data)), )
         with self.session(other_sess) as (conn, cur):
-            cur.execute(self.insert_sql % (
+            executed = cur.execute(self.insert_sql % (
                     tag, data,
                     schedule.strftime('timestamp \'%Y-%m-%d %H:%M:%S\'') if schedule else 'NULL', ))
-            res = cur.fetchone()
+            res = self.fetchone(cur if (not executed) else executed)
             cur.execute(self.notify_sql % (tag,))
             if conn: conn.commit()
             return res[0] if res else None
@@ -167,8 +177,8 @@ listen %s;
     def dequeue_item(self, tag, other_sess = None):
         tag = self.check_tag(tag)
         with self.session(other_sess) as (conn, cur):
-            cur.execute(self.select_sql % (tag,))
-            res = cur.fetchone()
+            executed = cur.execute(self.select_sql % (tag,))
+            res = self.fetchone(cur if (not executed) else executed)
             if res:
                 self.invoking_queue_id = res[0]
                 if ((0 < self.excepted_times_to_ignore) and
@@ -199,8 +209,8 @@ listen %s;
         interval    = self.LISTEN_TIMEOUT_INTERVAL_SECONDS
         while True:
             with self.session(None) as (conn, cur):
-                cur.execute(self.select_sql % (tag,))
-                res = cur.fetchone()
+                executed = cur.execute(self.select_sql % (tag,))
+                res = self.fetchone(cur if (not executed) else executed)
                 if res:
                     self.invoking_queue_id = res[0]
                     if not ((0 < self.excepted_times_to_ignore) and
@@ -222,8 +232,8 @@ listen %s;
                 conn.poll()
                 if conn.notifies:
                     notify = conn.notifies.pop()
-                    cur.execute(self.select_sql % (tag,))
-                    res = cur.fetchone()
+                    executed = cur.execute(self.select_sql % (tag,))
+                    res = self.fetchone(cur if (not executed) else executed)
                     if res:
                         self.invoking_queue_id = res[0]
                         if not ((0 < self.excepted_times_to_ignore) and
@@ -241,8 +251,8 @@ listen %s;
     def dequeue_immediate(self, tag, other_sess = None):
         tag = self.check_tag(tag)
         with self.session(other_sess) as (conn, cur):
-            cur.execute(self.select_sql % (tag,))
-            res = cur.fetchone()
+            executed = cur.execute(self.select_sql % (tag,))
+            res = self.fetchone(cur if (not executed) else executed)
             if res:
                 cur.execute(self.ack_sql % (res[0],))
                 if conn: conn.commit()
@@ -251,8 +261,8 @@ listen %s;
 
     def cancel(self, id, other_sess = None):
         with self.session(other_sess) as (conn, cur):
-            cur.execute(self.cancel_sql % (id,))
-            res = cur.fetchone()
+            executed = cur.execute(self.cancel_sql % (id,))
+            res = self.fetchone(cur if (not executed) else executed)
             if res and res[0]:
                 if conn: conn.commit()
                 return res[0]
@@ -262,14 +272,14 @@ listen %s;
         tag = self.check_tag(tag)
         schedule = (" and (schedule is null or schedule <= current_timestamp)" if ignore_scheduled else "")
         with self.session(other_sess) as (conn, cur):
-            cur.execute(self.list_sql % (tag, schedule, ))
-            res = cur.fetchall()
+            executed = cur.execute(self.list_sql % (tag, schedule, ))
+            res = self.fetchall(cur if (not executed) else executed)
             return res
 
     def count(self, tag, other_sess = None, ignore_scheduled = True):
         tag = self.check_tag(tag)
         schedule = (" and (schedule is null or schedule <= current_timestamp)" if ignore_scheduled else "")
         with self.session(other_sess) as (conn, cur):
-            cur.execute(self.count_sql % (tag, schedule, ))
-            res = cur.fetchone()[0]
+            executed = cur.execute(self.count_sql % (tag, schedule, ))
+            res = self.fetchone(cur if (not executed) else executed)[0]
             return int(res)
